// Name
// Student ID Number

// A constant that determines how far from the top
// of the screen the goal line should live.
int goal_line = 100;

// A Circle has a location for its centre, a radius, and
// a fill colour.
class Circle
{
  float x; // X coordinate of centre
  float y; // Y coordinate of centre
  float r; // Radius
  color c; // Fill colour

  // Construct a circle by passing in explicit
  // initial values for all the fields.
  Circle( float x, float y, float r, color c )
  {
    this.x = x;
    this.y = y;
    this.r = r;
    this.c = c;
  }

  // Determine whether this circle is past the goal
  // line.
  boolean goalAchieved()
  {
    // The circle is above the goal line if its bottom
    // edge (the Y coordinate of the centre plus the
    // radius) is above the goal line.
    return (y + r) <= goal_line;
  }

  // Tell this circle to draw itself.
  void draw()
  {
    fill( c );

    // If the circle is past the goal line, draw its
    // outline in black so that it's obvious that it
    // can't be moved any more.  Otherwise colour its
    // outline white.
    if ( goalAchieved() ) {
      stroke( 0, 0, 0 );
    } else {
      stroke( 0, 0, 255 );
    }

    ellipse( x, y, 2*r, 2*r );
  }

  // Check whether the given (x,y) position lies within
  // this circle, i.e., the usual hit test.
  boolean hitTest( int mx, int my )
  {
    // Add a special twist here: if the circle has
    // crossed the goal line, then the hit test always
    // fails.  That way, you'll never be able to move it
    // again.
    if ( !goalAchieved() ) {
      float d = dist( mx, my, x, y );
      return d <= r;
    } else {
      return false;
    }
  }
}

void setup()
{
  size( 160, 720 );
  colorMode( HSB, 255 );
}

void draw()
{
  background( 50 );

  // Draw the goal line.
  stroke( 30, 255, 200 );
  line( 10, goal_line, width - 10, goal_line );
}

// Determine whether all the circles have crossed the
// goal line (meaning that we're done), or if there are
// any left.
boolean checkAllDone( Circle[] arr )
{
  for ( Circle c : arr ) {
    if ( !c.goalAchieved() ) {
      return false;
    }
  }
  return true;
}