// Name
// Student ID

import controlP5.*;

ControlP5 ui;

// The source image that we're painting from.
PImage src;

// The additional width needed on the screen to
// accommodate all the UI controls.  This is a constant
// that you can modify to suit your interface.
int control_width = 250;

// The background colour to use in the control area.
int bg_colour = color( 100, 100, 100 );

// The current brush diameter.
float diameter;
// The current brush opacity.
float opacity;

// A flag to keep track of whether we're currently
// using the brush.  (You'd think that we could just
// use the built-in mousePressed variable, but we
// need to check whether we're pressing the mouse inside
// the control area or inside the painting area.)
boolean painting;

// One of the interface's buttons is provided for you.
Button quit_button;

void setup()
{
  // As of Processing 3, you are *required* to have the
  // call to size() be the first thing in setup(), and
  // a window's size must be based on two fixed numbers.
  // But we want the size of the window to depend on the
  // image, so....
  size( 800, 600 );

  src = loadImage( "image.jpg" );
  // ...after loading the image, we use this bit of
  // magic to *change* the size of the window to what
  // we want.
  surface.setSize( src.width + control_width, src.height );

  background( 255 );
  noStroke();

  // Choose initial values for the brush.
  opacity = 0.6;
  diameter = 50;

  // Now set up the user interface.  First create a ControlP5
  // object that manages all UI elements.
  ui = new ControlP5( this );

  // Now create all the UI elements one by one.
  quit_button = ui.addButton( "Quit" )
    .setPosition( src.width + control_width/2 - 25, height/2 )
    .setSize( 50, 30 );
    
  // TODO -- you'll want to set up a lot more widgets here.

  painting = false;
}

// Draw a solid-coloured backdrop that will sit under the
// UI area of the window.
void drawControlRect()
{
  fill( bg_colour );
  rect( src.width, 0, control_width, height );
}

void draw()
{
  // Check if the user is using a brush as we start
  // drawing.  If so, add a stroke.
  if ( painting ) {
    // Set the colour of the brush to be the colour
    // of the source image's pixel at this point.
    fill( src.get( mouseX, mouseY ), opacity * 255.0 );
    ellipse( mouseX, mouseY, diameter, diameter );
  }

  // Now draw the UI backdrop *on top* of any new
  // strokes, to stencil them out if needed.
  drawControlRect();
}

void mousePressed()
{
  // Only start painting if we're in the painting area.
  // If we're in the control area, let ControlP5 decide if
  // we're interacting with a widget.
  painting = (mouseX < src.width);
}

void mouseReleased()
{
  painting = false;
}

// Save just the painted part of the screen, and not the
// control area too.  Do this with the help of Processing's
// get() function.
void savePainting()
{
  PImage out = get( 0, 0, src.width, src.height );
  out.save( "output.png" );
}

// Respond to any events generated from UI widgets.
void controlEvent( ControlEvent ev )
{
  if ( ev.isFrom( quit_button ) ) {
    exit();
  }
}